local E = AleaUI
local Skins = AleaUI:Module("Skins")

local _G = _G

local default_background_color = Skins.default_background_color
local default_border_color = Skins.default_border_color
local default_font = Skins.default_font
local default_texture = Skins.default_texture

local default_button_background = Skins.default_button_background
local default_button_border 	= Skins.default_button_border
local default_border_color_dark = Skins.default_border_color_dark

local ITEM_LEVEL_PATTERN = ITEM_LEVEL:gsub('%%d', '(%%d+)')
local ITEM_UPGRADE_TOOLTIP_FORMAT_PATTERN = ITEM_UPGRADE_TOOLTIP_FORMAT:gsub('%%d', '(%%d+)')

local varName = 'inspect'
AleaUI.default_settings.skins[varName] = true

local function InitInspect()
	if not AleaUI.db.skins.enableAll then return end
	if not AleaUI.db.skins[varName] then return end

	local Updater
	
	local GetItemItemLevel
	do
		local hidegametooltip = CreateFrame("Frame")
		hidegametooltip:Hide()
		local gametooltip = CreateFrame("GameTooltip", "AleaUIInpectFrameItemLevelParserGameToolTip", nil, "GameTooltipTemplate");
		gametooltip:SetOwner( hidegametooltip,"ANCHOR_NONE");
		gametooltip:SetScript('OnTooltipAddMoney', function()end)
		gametooltip:SetScript('OnTooltipCleared', function()end)
		gametooltip:SetScript('OnHide', function()end)
		gametooltip:SetScript('OnTooltipSetDefaultAnchor',function()end)
	
		
		function GetItemItemLevel( unit, id )
			gametooltip:ClearLines()
			gametooltip:SetInventoryItem( unit, id )
			
			local itemLevel, itemUpgrade, itemUpgradeTotal = nil, nil, nil
			
			for i=1, gametooltip:NumLines() do		
				local line = _G[gametooltip:GetName().."TextLeft"..i]:GetText()			
				if line then
					local ilvl = string.match(line, ITEM_LEVEL_PATTERN)	
					local grade1, grade2 = string.match(line, ITEM_UPGRADE_TOOLTIP_FORMAT_PATTERN)
					
					if ilvl and tonumber(ilvl) then
						itemLevel = tonumber(ilvl)
					end			
					
					if grade1 and grade2 and tonumber(grade1) and tonumber(grade2) then
						itemUpgrade = tonumber(grade1)
						itemUpgradeTotal = tonumber(grade2)
					end
				end
			end

			return itemLevel, itemUpgrade, itemUpgradeTotal
		end
	end
	
	local countToTotalIlvl = {
		[1] = true,
		[2] = true,
		[3] = true,
		[15] = true,
		[5] = true,
		[9] = true,
		
		[10] = true,
		[6] = true,
		[7] = true,
		[8] = true,
		[11] = true,
		[12] = true,
		[13] = true,
		[14] = true,
		
		[16] = true,
		[18] = true,
		
		[17] = false, -- offhand
	}
	local buttonList = {
		'InspectMainHandSlot', 'MainHand', true,
		'InspectSecondaryHandSlot', 'SecondaryHand', true,
		'InspectHeadSlot', 'Head', true,
		'InspectNeckSlot', 'Neck', true,
		'InspectShoulderSlot', 'Shoulder', true,
		'InspectBackSlot', 'Chest', true, 
		'InspectChestSlot', 'Chest', true,
		'InspectShirtSlot', 'Shirt', false,
		'InspectTabardSlot', 'Tabard', false,
		'InspectWristSlot', 'Wrists', true,
		'InspectHandsSlot', 'Hands', true,
		'InspectWaistSlot', 'Waist', true,
		'InspectLegsSlot', 'Legs', true,
		'InspectFeetSlot', 'Feet', true,
		'InspectFinger0Slot', 'Finger', true,
		'InspectFinger1Slot', 'Finger', true,
		'InspectTrinket0Slot', 'Trinket', true,
		'InspectTrinket1Slot', 'Trinket', true,
	}
	
	local function GetTotalItemLvl(self, elapsed)
		self.elapsed = ( self.elapsed or 0 ) + elapsed
		if (self.elapsed) < 0.5 then return end
		self.elapsed = 0
		
		local unit = InspectFrame.unit
		local total = 0
		local hide = true
		
		self.attept = self.attept + 1
		
		for id, enabled in pairs(countToTotalIlvl) do
			
			local itemLevel, itemUpgrade, itemUpgradeTotal = GetItemItemLevel( unit, id )
		
			if itemLevel then
				if enabled then
					total = total + itemLevel
				end
				
				local buttonList_id2 = 1
				
				for i = 1, #buttonList/3 do
		
					if buttonList[buttonList_id2+2] and _G[buttonList[buttonList_id2]]:GetID() == id then			
						_G[buttonList[buttonList_id2]].ilvl:SetText( format( ( itemUpgrade and '%d\n%d/%d' or '%d' ), itemLevel, itemUpgrade, itemUpgradeTotal))	
					end
					
					buttonList_id2 = buttonList_id2 + 3
				end
			else
				hide = false
			end
		end
		
		if hide or self.attept == 5 then
			self:Hide()
		end
	
		InspectFrame.tilvl:SetText('ILvL:'..floor(total/15))
	end
	
	Updater = CreateFrame('Frame', nil, InspectFrame)
	Updater:SetScript('OnUpdate', GetTotalItemLvl)
	Updater:Hide()
	
	InspectFrame:HookScript('OnShow', function()
	
		local buttonList_id = 1
	
		for i=1, #buttonList/3 do
			if _G[buttonList[buttonList_id]].ilvl then			
				_G[buttonList[buttonList_id]].ilvl:SetText('')
			end		
			buttonList_id = buttonList_id + 3
		end
		
		InspectFrame.tilvl:SetText('')
		
		Updater.attept = 0
		Updater:Show()
	end)
	
	InspectFrame.tilvl = InspectFrame:CreateFontString(nil, "ARTWORK", "GameFontWhite")
	InspectFrame.tilvl:SetFont(STANDARD_TEXT_FONT, 12, "OUTLINE")
	InspectFrame.tilvl:SetPoint("BOTTOMLEFT", InspectFrame, "BOTTOMLEFT", 3, 2)


	_G['InspectFrame']:StripTextures()
	_G['InspectFrameInset']:StripTextures()

	Skins.ThemeBackdrop('InspectFrame')
	Skins.ThemeFrameRing('InspectFrame')
	
	for i=1, 4 do
		Skins.ThemeTab('InspectFrameTab'..i)
	end
	
	-- frame.items[i].ilvl:SetText( format( ( itemUpgrade and '%d\n%d/%d' or '%d' ), itemLevelFull, itemUpgrade, itemUpgradeTotal))	
	
	local buttonList_id = 1
	
	for i=1, #buttonList/3 do
		Skins.ThemeItemButton(buttonList[buttonList_id], buttonList[buttonList_id+1])
		if buttonList[buttonList_id+2] then			
			_G[buttonList[buttonList_id]].ilvl = _G[buttonList[buttonList_id]]:CreateFontString(nil, "ARTWORK", "GameFontWhite")
			_G[buttonList[buttonList_id]].ilvl:SetFont(E.media.default_font2, 12, "OUTLINE")
			_G[buttonList[buttonList_id]].ilvl:SetPoint("BOTTOMRIGHT", _G[buttonList[buttonList_id]], "BOTTOMRIGHT")
		end		
		buttonList_id = buttonList_id + 3
	end
	--[==[
	Skins.ThemeItemButton('InspectMainHandSlot', 'MainHand')
	Skins.ThemeItemButton('InspectSecondaryHandSlot', 'SecondaryHand')
	Skins.ThemeItemButton('InspectHeadSlot', 'Head')
	Skins.ThemeItemButton('InspectNeckSlot', 'Neck')
	Skins.ThemeItemButton('InspectShoulderSlot', 'Shoulder')
	Skins.ThemeItemButton('InspectBackSlot', 'Chest')
	Skins.ThemeItemButton('InspectChestSlot', 'Chest')
	Skins.ThemeItemButton('InspectShirtSlot', 'Shirt')
	Skins.ThemeItemButton('InspectTabardSlot', 'Tabard')
	Skins.ThemeItemButton('InspectWristSlot', 'Wrists')
	Skins.ThemeItemButton('InspectHandsSlot', 'Hands')
	Skins.ThemeItemButton('InspectWaistSlot', 'Waist')
	Skins.ThemeItemButton('InspectLegsSlot', 'Legs')
	Skins.ThemeItemButton('InspectFeetSlot', 'Feet')
	Skins.ThemeItemButton('InspectFinger0Slot', 'Finger')
	Skins.ThemeItemButton('InspectFinger1Slot', 'Finger')
	Skins.ThemeItemButton('InspectTrinket0Slot', 'Trinket')
	Skins.ThemeItemButton('InspectTrinket1Slot', 'Trinket')
	]==]
	
	
	
	Skins.ThemeButton(InspectPaperDollFrame.ViewButton)
end

AleaUI:OnInit2(function()
	if InspectFrame_Show then
		C_Timer.After(2, InitInspect)
	else
		AleaUI:OnAddonLoad('Blizzard_InspectUI', InitInspect)
	end
end)
